/*
 * This class file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType.calculation;

import java.util.List;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationExpression;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;

/**
 * This class is the missing link between a VariableDeclarationExpression and a 
 * VariableDeclarationStatement. Both contain the methods
 *  
 * - getType()  
 * - fragments()
 * 
 * However, in both classes they are declared from scratch. 
 * 
 * This class is the missing superclass of the two.
 * 
 * @version: $Date: 2007/07/28 12:12:55 $Author: kegel $Revision: 1.1 $
 * @author Philip Mayer
 */
public class VariableDeclarationExpressionStatement {

	ASTNode node;

	/**
	 * Constructor for VariableDeclarationExpressionStatement.
	 * 
	 * @param expressionORStatement ASTNode
	 */
	public VariableDeclarationExpressionStatement(ASTNode expressionORStatement) {

		node = expressionORStatement;
	}

	/**
	 * Returns the type.
	 * @return Type
	 */
	public Type getType() {

		if (node instanceof VariableDeclarationExpression)
			return ((VariableDeclarationExpression) node).getType();
		if (node instanceof VariableDeclarationStatement)
			return ((VariableDeclarationStatement) node).getType();
		return null;
	}

	/**
	 * Returns the fragments.
	 * 
	 * @return List
	 */
	public List fragments() {

		if (node instanceof VariableDeclarationExpression)
			return ((VariableDeclarationExpression) node).fragments();
		if (node instanceof VariableDeclarationStatement)
			return ((VariableDeclarationStatement) node).fragments();
		return null;
	}

	/**
	 * Returns the node itself.
	 * 
	 * @return ASTNode
	 */
	public ASTNode getASTNode() {

		return node;
	}

	/**
	 * Method toString.
	 * @return String
	 */
	public String toString() {

		if (node != null) {
			String x = (node.toString().length() > 2) ? node.toString().substring(0, node.toString().length() - 2) : node.toString();
			return "VariableDeclarationExpression[" + x + "]";
		} else
			return "VariableDeclarationExpression[-node is null-]";
	}
}