/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.internal.constraintModel.variables;

import org.eclipse.jdt.internal.corext.refactoring.typeconstraints.types.TType;
import org.eclipse.jdt.internal.corext.refactoring.typeconstraints.types.TypeVariable;
import org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ConstraintVariable2;

/**
 * A variable representing the bound of a type variable. Can be used for
 * generic types and generic methods.
 * 
 * @author Hannes Kegel
 *
 */
public class BoundVariable extends ConstraintVariable2 {
    private String typeVarKey;
    private int index;

    /**
     * The only constructor.
     * 
     * @param type the bound type
     * @param typeVariable the type variable this is a bound for 
     * @param index the index of bound (not the type variable)
     */
    public BoundVariable(TType type, TypeVariable typeVariable, int index) {
        super(type);
        typeVarKey = typeVariable.getBindingKey();
        this.index = index;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + index;
        result = prime * result + ((typeVarKey == null) ? 0 : typeVarKey.hashCode());
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof BoundVariable)) {
            return false;
        }
        final BoundVariable other = (BoundVariable) obj;
        if (index != other.index) {
            return false;
        }
        if (typeVarKey == null) {
            if (other.typeVarKey != null) {
                return false;
            }
        } else if (!typeVarKey.equals(other.typeVarKey)) {
            return false;
        }
        return true;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ConstraintVariable2#toString()
     */
    @Override
    public String toString() {
        return "[Bound(" + index + "," + typeVarKey + ") " + super.toString() + "]";
    }
    
    
    
    

}
