/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.internal.ui;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.ltk.ui.refactoring.UserInputWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.intoJ.inferType3.InferTypePlugin;
import org.intoJ.inferType3.InferTypeRefactoring;
import org.intoJ.inferType3.internal.typeCalc.AccessSet;

/**
 * The single custom wizard page of the refactoring wizard.
 * 
 * @author Hannes Kegel
 *
 */
class ITInputPage extends UserInputWizardPage {

    private static final int WIDTH = 550;
    private static final String PAGE_NAME= "InferTypeInputPage";

    private final InferTypeRefactoring refactoring;

    private boolean initialized = false;
    private Composite newComposite;
    		
	/**
     * The single constructor.
     * 
	 * @param inferTypeWizard the wizard
	 */
	public ITInputPage(InferTypeWizard inferTypeWizard) {
		super(PAGE_NAME);
        refactoring = (InferTypeRefactoring)inferTypeWizard.getRefactoring();
	}

    /* (non-Javadoc)
     * @see org.eclipse.ltk.ui.refactoring.UserInputWizardPage#setVisible(boolean)
     */
    @Override
    public void setVisible(boolean visible) {
        if (visible && !initialized) {
            initialized = true;
            try {
                final InferTypeRefactoring inferTypeProcessor = ((InferTypeRefactoring)getRefactoring());
                Composite composite = (Composite)getControl();
                String existingName = inferTypeProcessor.getFittingExistingTypeName();
                String originalName = inferTypeProcessor.getTypeHierarchy()
            	.getOriginalSelectedType().getTypeName();
                String newPackageName = inferTypeProcessor.getNewPackageName();
                boolean existingInPreferredPackage = true;
                
                if (newPackageName != null && existingName != null
                		&& (!existingName.startsWith(newPackageName)
                		|| existingName.lastIndexOf('.') != newPackageName.length())) {
                	//type at existingName is in the package at newPackageName
                	existingInPreferredPackage = false;
                }

                if (existingName != null && !existingName.equals(originalName)
                		&& (newPackageName == null || existingInPreferredPackage)) {
                    createExistingComposite(composite, existingName);
                    createNewComposite(composite, true, inferTypeProcessor);
                    setNewEnabled(false);
                } else if (existingName != null && existingName.equals(originalName)) {
                	createNewComposite(composite, true, inferTypeProcessor);
                	inferTypeProcessor.setForceNew(true);
                } else {
                    createNewComposite(composite, false, inferTypeProcessor);                        
                }
                
                addOptionGroup(composite);
                if (InferTypePlugin.getDefault().getPreferenceStore().getBoolean(InferTypePlugin.PREFNAME_SHOW_EXPORT)) {
                    addExportGroup(composite);
                }                    
                                    
                Dialog.applyDialogFont(composite);
                
            } catch (JavaModelException e) {
                e.printStackTrace();
                setErrorMessage("Could not determine access sets");
            }
        }
        super.setVisible(visible);
    }
    

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        initializeDialogUnits(parent);
        Composite composite= new Composite(parent, SWT.NONE);
        setControl(composite);
        GridLayout gridLayout = new GridLayout(1, false);
        gridLayout.verticalSpacing = 8;
        composite.setLayout(gridLayout);
    }
    

    private void addOptionGroup(Composite composite) {
        Group group = new Group(composite, SWT.NONE);
        group.setText("Options");
        GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
        gridData.verticalIndent = 8; 
        group.setLayoutData(gridData);
        
        GridLayout gridLayout2 = new GridLayout(1, false);
        gridLayout2.verticalSpacing = 8;
        gridLayout2.marginTop = 3;
        group.setLayout(gridLayout2);

// not very useful for a single refactoring run and leads to a clunky error handling. 
// The user can see in the preview what would have to be changed if it is important.
        
//            final Button changeTypeArgsButton = new Button(group, SWT.CHECK);
//            gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
//            changeTypeArgsButton.setLayoutData(gridData);
//            changeTypeArgsButton.setText(" change type arguments if necessary");
//            changeTypeArgsButton.setSelection(((InferTypeRefactoring)getRefactoring()).isChangeTypeArguments());
//            changeTypeArgsButton.addSelectionListener(new SelectionAdapter() {
//                public void widgetSelected(SelectionEvent e) {
//                    ((InferTypeRefactoring)getRefactoring()).setChangeTypeArguments(changeTypeArgsButton.getSelection());
//                }
//            });

      final Button redclaredDEButton = new Button(group, SWT.CHECK);
        gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
        redclaredDEButton.setLayoutData(gridData);
        redclaredDEButton.setText(" redeclare the selected declaration element");
        redclaredDEButton.setSelection(((InferTypeRefactoring) getRefactoring()).isRedeclareDEs());
        redclaredDEButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ((InferTypeRefactoring) getRefactoring()).setRedeclareDEs(redclaredDEButton.getSelection());
            }
        });

        final Button noAbstractClassesButton = new Button(group, SWT.CHECK);
        gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
        noAbstractClassesButton.setLayoutData(gridData);
        noAbstractClassesButton.setText(" don't generate abstract classes");
        noAbstractClassesButton.setSelection(((InferTypeRefactoring)getRefactoring()).isNoAbstractClasses());
        noAbstractClassesButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ((InferTypeRefactoring)getRefactoring()).setNoAbstractClasses(noAbstractClassesButton.getSelection());
            }
        });
        

    }

    private void addExportGroup(Composite composite) {
        Group group = new Group(composite, SWT.NONE);
        group.setText("Constraint Graph Export");
        GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
        gridData.verticalIndent = 8; 
        group.setLayoutData(gridData);
        
        GridLayout gridLayout2 = new GridLayout(2, false);
        gridLayout2.verticalSpacing = 8;
        group.setLayout(gridLayout2);
        
        Label label = new Label(group, SWT.WRAP);
        gridData = new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1);
        gridData.widthHint = WIDTH;
        label.setLayoutData(gridData);
        label.setText("You can export the type constraint graph to a gml file. " + 
                "This option is only relevant if you are interested in how 'Infer Type' works.");
        
        final Button chkTypeArgumentEdges = new Button(group, SWT.CHECK);            
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, false);
        chkTypeArgumentEdges.setLayoutData(gridData);
        chkTypeArgumentEdges.setText(" include type argument edges");

        final Button cmdExport = new Button(group, SWT.NONE);
        gridData = new GridData(SWT.RIGHT, SWT.BOTTOM, true, false, 1, 2);
        cmdExport.setLayoutData(gridData);
        cmdExport.setText("    Export ...    ");
        
        final Button chkOnlyTraversed = new Button(group, SWT.CHECK);            
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, false);
        chkOnlyTraversed.setLayoutData(gridData);
        chkOnlyTraversed.setText(" export traversed nodes only");
        
        cmdExport.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                FileDialog fileDialog = new FileDialog(getShell(), SWT.SAVE);
                fileDialog.setFilterExtensions(new String[] { "*.gml" });
                String fileName = fileDialog.open();
                if (fileName != null) {
                    try {
                        ((InferTypeRefactoring)getRefactoring()).exportGraph(fileName, chkTypeArgumentEdges.getSelection(), chkOnlyTraversed.getSelection());
                    } catch (Exception ex) {
                        MessageBox messageBox = new MessageBox(getShell(), SWT.ICON_ERROR);
                        messageBox.setText("Infer Type");
                        messageBox.setMessage("Error exporting type constraint graph:\n" + ex);
                        messageBox.open();
                        InferTypePlugin.getDefault().getLog().log(new Status(IStatus.ERROR, InferTypePlugin.PLUGIN_ID, IStatus.ERROR, 
                                "Error exporting type constraint graph", ex));
                    }
                }
            }
        });
        
    }

    private void createNewComposite(Composite composite, boolean existingPossible , final InferTypeRefactoring inferTypeProcessor) throws JavaModelException {
        newComposite = new Composite(composite, SWT.NONE);
        GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        newComposite.setLayoutData(gridData);

        GridLayout gridLayout2 = new GridLayout(2, false);
        gridLayout2.verticalSpacing = 8;
        gridLayout2.marginHeight = 0;
        gridLayout2.marginWidth = 0;
        gridLayout2.marginLeft = existingPossible ? 20 : 0;
        gridLayout2.marginRight= 0;
        gridLayout2.marginTop = 0;
        gridLayout2.marginBottom = 0;
        newComposite.setLayout(gridLayout2);

        Label label = new Label(newComposite, SWT.WRAP);
        gridData = new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1);
        gridData.widthHint = WIDTH;
        label.setLayoutData(gridData);
        label.setText("The initially selected methods are used on the reference and must be contained in the new type. " + 
                "You can add additional methods to the new type below.");
                    
        CheckboxTreeViewer checkBoxTreeViewer = new CheckboxTreeViewer(newComposite);

        AccessSet accessSet = inferTypeProcessor.getAccessSet();            
        ITContentProvider contentProvider = new ITContentProvider(accessSet);
        checkBoxTreeViewer.setContentProvider(contentProvider);
        checkBoxTreeViewer.setLabelProvider(new ITLabelProvider());
        gridData = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
        gridData.heightHint = existingPossible ? 200 : 250;
        gridData.widthHint = WIDTH;
        checkBoxTreeViewer.getControl().setLayoutData(gridData);
        checkBoxTreeViewer.setInput(inferTypeProcessor.getTypeHierarchy());
        checkBoxTreeViewer.setCheckedElements(accessSet.getMembers().toArray());
        checkBoxTreeViewer.addCheckStateListener(new CheckStateListener(refactoring));
        checkBoxTreeViewer.expandAll();
        Object[] elements = ((ITContentProvider)checkBoxTreeViewer.getContentProvider()).getElements(inferTypeProcessor.getTypeHierarchy());
        if (elements.length > 0) {
            checkBoxTreeViewer.reveal(elements[0]);
        }

        String otherMemberDescription = refactoring.getOtherMemberDescription();
        if (otherMemberDescription != null) {
            label = new Label(newComposite, SWT.WRAP);
            gridData = new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1);
            gridData.widthHint = WIDTH;
            label.setLayoutData(gridData);
            label.setText(otherMemberDescription);
        }
        

        label = new Label(newComposite, 0);
        gridData = new GridData(SWT.LEFT, SWT.CENTER, false, false);
        label.setLayoutData(gridData);
        label.setText("Type name: ");
        
        final Text textTypeName = new Text(newComposite, SWT.SINGLE | SWT.LEFT | SWT.BORDER);
        gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
        textTypeName.setLayoutData(gridData);
        textTypeName.setText(inferTypeProcessor.getNewTypeName());    
        textTypeName.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                inferTypeProcessor.setNewTypeName(textTypeName.getText());
            }
            
        });
        
    }

    private void createExistingComposite(Composite composite, String existingName) {
        Composite existingComposite = new Composite(composite, SWT.NONE);
        GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
        existingComposite.setLayoutData(gridData);

        GridLayout gridLayout2 = new GridLayout(1, false);
        gridLayout2.verticalSpacing = 8;
        gridLayout2.marginHeight = 0;
        gridLayout2.marginWidth = 0;
        gridLayout2.marginLeft = 0;
        gridLayout2.marginRight= 0;
        gridLayout2.marginTop = 0;
        gridLayout2.marginBottom = 0;
        existingComposite.setLayout(gridLayout2);
        
        Label label = new Label(existingComposite, SWT.WRAP);
        gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
        label.setLayoutData(gridData);
        label.setText("An existing type in the hierarchy could be used as a minimal type:");

        gridData = new GridData(SWT.FILL, SWT.CENTER, true, false);
        Button rdoExisting = new Button(existingComposite, SWT.RADIO);
        rdoExisting.setLayoutData(gridData);
        rdoExisting.setText(" use type " + existingName);
        rdoExisting.setSelection(true);
        rdoExisting.addSelectionListener(new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {
                widgetSelected(e);
            }

            public void widgetSelected(SelectionEvent e) {
                setNewEnabled(false);
            }
            
        });
        
        Button rdoNew = new Button(existingComposite, SWT.RADIO);
        rdoNew.setLayoutData(gridData);
        rdoNew.setText(" create a new type");       
        rdoNew.addSelectionListener(new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {
                widgetSelected(e);
            }

            public void widgetSelected(SelectionEvent e) {
                setNewEnabled(true);
            }
            
        });
    }

    private void setNewEnabled(boolean b) {
        final InferTypeRefactoring inferTypeProcessor = ((InferTypeRefactoring)getRefactoring());
        inferTypeProcessor.setForceNew(b);
        for(Control control : newComposite.getChildren()) {
            control.setEnabled(b);
        }
    }                        
    
}