/*
 * This class file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType.model.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * 
 * Superclass for objects which can inform other objects about changes (Observer pattern).
 * Makes use of the java.beans notification classes.
 * 
 * @version: $Date: 2007/09/09 19:40:25 $Author: kegel $Revision: 1.1 $
 * @author Philip Mayer
 */
public class InformingObject {

	public static final String TYPE_NAME = "TYPE_NAME";
	public static final String TYPE_TYPE = "TYPE_TYPE";

	protected transient PropertyChangeSupport listeners = new PropertyChangeSupport(this);

	/**
	 * Method addPropertyChangeListener.
	 * @param l PropertyChangeListener
	 */
	public void addPropertyChangeListener(PropertyChangeListener l) {

		listeners.addPropertyChangeListener(l);
	}

	/**
	 * Method removePropertyChangeListener.
	 * @param l PropertyChangeListener
	 */
	public void removePropertyChangeListener(PropertyChangeListener l) {

		listeners.removePropertyChangeListener(l);
	}

	/**
	 * Informs the listeners about a property change.
	 * 
	 * @param prop
	 * @param old
	 * @param newValue
	 */
	protected void firePropertyChange(String prop, Object old, Object newValue) {

		listeners.firePropertyChange(prop, old, newValue);
	}

}
