/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoj.riwd;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
import org.eclipse.jdt.core.dom.AnnotationTypeDeclaration;
import org.eclipse.jdt.core.dom.EnumDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
import org.eclipse.jdt.core.dom.SuperFieldAccess;
import org.eclipse.jdt.core.dom.SuperMethodInvocation;
import org.eclipse.jdt.internal.corext.dom.HierarchicalASTVisitor;

public class SuperCallVisitor extends HierarchicalASTVisitor {
	
	private List<SuperConstructorInvocation> superConstructorInvocationList = new ArrayList<SuperConstructorInvocation>();
	private List<SuperFieldAccess> superFieldAccessList = new ArrayList<SuperFieldAccess>();
	private List<SuperMethodInvocation> superMethodInvocationList = new ArrayList<SuperMethodInvocation>();
	private List<MethodDeclaration> constructorWithoutSuperList = new ArrayList<MethodDeclaration>();

	private Stack<MethodDeclaration> currentConstructors = new Stack<MethodDeclaration>();

	private boolean first = true;
	@Override
	public boolean visit(AbstractTypeDeclaration node) {
		if (first) {
			first = false;
			return super.visit(node);
		} else {
			return false;
		}
	}

	@Override
	public boolean visit(AnnotationTypeDeclaration node) {
		return false;
	}

	@Override
	public boolean visit(EnumDeclaration node) {
		return false;
	}

	@Override
	public boolean visit(MethodDeclaration node) {
		if (node.isConstructor()) {
			currentConstructors.push(node);
		}
		return super.visit(node);
	}

	@Override
	public void endVisit(MethodDeclaration node) {
		if (node.isConstructor()) {
			MethodDeclaration currentConstructor = currentConstructors.pop();
			if (currentConstructor != null) {
				constructorWithoutSuperList.add(currentConstructor);
			}
		}
		super.endVisit(node);
	}

	@Override
	public boolean visit(SuperConstructorInvocation node) {
		superConstructorInvocationList.add(node);
		currentConstructors.pop();
		currentConstructors.push(null);
		return super.visit(node);
	}

	@Override
	public boolean visit(SuperFieldAccess node) {
		superFieldAccessList.add(node);
		return super.visit(node);
	}

	@Override
	public boolean visit(SuperMethodInvocation node) {
		superMethodInvocationList.add(node);
		return super.visit(node);
	}

	public List<SuperConstructorInvocation> getSuperConstructorInvocationList() {
		return superConstructorInvocationList;
	}

	public List<SuperFieldAccess> getSuperFieldAccessList() {
		return superFieldAccessList;
	}

	public List<SuperMethodInvocation> getSuperMethodInvocationList() {
		return superMethodInvocationList;
	}

	public List<MethodDeclaration> getConstructorWithoutSuperList() {
		return constructorWithoutSuperList;
	}
	
}    
    
