/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoj.riwd.ui;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.intoJ.inferType3.internal.constraintModel.util.Scope;

/**
 * This dialog is used to ask the user for the desired scope (cu, package, package and subpackage, project)
 * of the import refactoring.
 * 
 * @author Hannes Kegel
 *
 */
public class ImportScopeDialog extends MessageDialog {

    
    /**
     * Show a dialog that asks the user for the desired scope.
     * 
     * @param parentShell the parent shell
     * @return the selected scope or null if the dialog was cancelled
     */
    public static Scope askForScope(final Shell parentShell) {
        final Scope[] result = new Scope[1];
        parentShell.getDisplay().syncExec(new Runnable() {
            public void run() {
                ImportScopeDialog importScopeDialog = new ImportScopeDialog(parentShell);
                importScopeDialog.open();
                result[0] = importScopeDialog.getScope();
            }
        });
        return result[0];
    }
    
    private Scope scope;
    
    private Button cuScopeButton;
    private Button packageScopeButton;
    private Button subScopeButton;
    private Button projectScopeButton;

    /**
     * The single constructor.
     * 
     * @param parentShell the parent shell
     */
    private ImportScopeDialog(Shell parentShell) {
        super(parentShell, "Infer Type", null, "What scope do you want to create a single type for?", 
                QUESTION, new String[] { "OK", "Cancel" }, 0);
        setBlockOnOpen(true);
    }
    
    /**
     * Retrieves the selection.
     * 
     * @return the selected scope
     */
    private Scope getScope() {
        return scope;
    }
    
    protected void buttonPressed(int buttonId) {
        if (buttonId == 0) {
            if (packageScopeButton.getSelection()) {
                scope = Scope.PACKAGE;            
            } else if (subScopeButton.getSelection()) {
                scope = Scope.PACKAGE_INCLUDING_SUBPACKAGES;                        
            } else if (projectScopeButton.getSelection()) {
                scope = Scope.PROJECT;                        
            } else {
                scope = Scope.COMPILATION_UNIT; 
            }
        }
        super.buttonPressed(buttonId);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.MessageDialog#createCustomArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createCustomArea(Composite parent) {
        Composite composite = new Composite(parent, SWT.NONE);

        GridLayout gridLayout2 = new GridLayout(1, false);
        gridLayout2.verticalSpacing = 6;
        gridLayout2.marginTop = 0;
        gridLayout2.marginBottom = 20;
        gridLayout2.marginLeft = 70;
        gridLayout2.marginRight= 0;
        composite.setLayout(gridLayout2);

        cuScopeButton = new Button(composite, SWT.RADIO);
        GridData gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
        cuScopeButton.setLayoutData(gridData);
        cuScopeButton.setText(" this compilation unit");
        cuScopeButton.setSelection(true);

        packageScopeButton = new Button(composite, SWT.RADIO);
        gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
        packageScopeButton.setLayoutData(gridData);
        packageScopeButton.setText(" this package");
        packageScopeButton.setSelection(false);

        subScopeButton = new Button(composite, SWT.RADIO);
        gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
        subScopeButton.setLayoutData(gridData);
        subScopeButton.setText(" this package and its sub-packages");
        subScopeButton.setSelection(false);

        projectScopeButton = new Button(composite, SWT.RADIO);
        gridData = new GridData(SWT.LEFT, SWT.TOP, true, false);
        projectScopeButton.setLayoutData(gridData);
        projectScopeButton.setText(" this project");
        projectScopeButton.setSelection(false);

        return composite;
    }
    

}
