/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoj.riwd.ui;

import org.intoJ.inferType3.internal.typeCalc.hierarchy.BindingWrapper;
import org.intoJ.inferType3.internal.typeCalc.hierarchy.TCType;

/**
 * A wrapper for a {@link BindingWrapper} that has a parent type
 * and acts as a leaf in the tree. More than one WrapperLeaf can exist
 * for a BindingWrapper.
 * 
 * @author Hannes Kegel
 *
 */
class WrapperLeaf {
    private TCType parent;
    private BindingWrapper wrapper;

    /**
     * The single constructor.
     * 
     * @param parent the parent defining the method
     * @param wrapper the binding wrapper
     */
    public WrapperLeaf(TCType parent, BindingWrapper wrapper) {
        this.parent = parent;
        this.wrapper = wrapper;
    }

    /**
     * Returns the parent defining the binding
     * @return the parent
     */
    public TCType getParent() {
        return parent;
    }

    /**
     * Returns the binding wrapper
     * @return the wrapper
     */
    public BindingWrapper getWrapper() {
        return wrapper;
    }       

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return getWrapper().toString();
    }

}
